import os

try:
    from json import loads as json
except ImportError:
    from simplejson import loads as json

try:
    from yaml import load as yaml
except ImportError as e:
    error = e
    def yaml(value):
        raise error

from .compat import StringIO
from .compat import SafeConfigParser
from .compat import string_type
from .exceptions import ParserNotFound


def configparser(data):
    result = {}

    strio = StringIO()
    strio.write(data)
    strio.seek(0)

    parser = SafeConfigParser()
    parser.readfp(strio)

    for section in parser.sections():
        section_data = {}

        for key, value in parser.items(section):
            section_data[key] = value

        result[section] = section_data

    return result


def parser_from_parser_or_filename(parser, filename):

    if callable(parser):
        return parser

    if isinstance(parser, string_type):
        return parser_for_ext(parser)

    if filename:
        return parser_for_filename(filename)

    return None


def parser_for_filename(filename):
    ext = os.path.splitext(filename)[1][1:].strip().lower()

    if ext:
        return parser_for_ext(ext)
    else:
        return lambda x: x


def parser_for_ext(ext):

    mapping = {
    'yml': yaml,
    'yaml': yaml,
    'json': json,
    'ini': configparser,
    'configparser': configparser
    }

    result = mapping.get(ext.lower(), None)

    if result:
        return result

    raise ParserNotFound(ext)



from collections import deque
from .loaders import load_file
from .loaders import load_env_path
from .loaders import load_env_keys
from .loaders import load_locations
from .loaders import load_python
from .loaders import load_env_python
from .parsers import parser_from_parser_or_filename


class AttributeDict(dict):
    '''
    Pulled directly from lexicon:
    https://pypi.python.org/pypi/lexicon
    https://github.com/bitprophet/lexicon
    '''
    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            # to conform with __getattr__ spec
            raise AttributeError(key)

    def __setattr__(self, key, value):
        self[key] = value

    def __delattr__(self, key):
        del self[key]

    def __getitem__(self, key):
        '''
        Not included from lexicon. We want everything returned
        from an attribute dict to also return an attribute dict
        if it's a dict
        '''
        result = dict.__getitem__(self, key)

        try:
            return AttributeDict(result)
        except (TypeError, ValueError):
            return result


class Location(object):

    @classmethod
    def from_python(cls, path, parser=None):
        obj = Location(path, loader=load_python, parser=parser)
        return obj

    @classmethod
    def from_env_python(cls, path, parser=None):
        obj = Location(path, loader=load_env_python, parser=parser)
        return obj

    @classmethod
    def from_path(cls, path, parser=None):
        parser = parser_from_parser_or_filename(parser, path)
        obj = Location(path, loader=load_file, parser=parser)
        return obj

    @classmethod
    def from_env_path(cls, key, parser=None):
        value = load_env_keys((key,))
        try:
            parser = parser_from_parser_or_filename(parser, value[key])
            obj = Location(key, loader=load_env_path, parser=parser)
        except KeyError:
            obj = {}

        return obj

    @classmethod
    def from_env_keys(cls, keys, parser=None):
        obj = Location(keys, loader=load_env_keys, parser=parser)
        return obj

    @classmethod
    def from_dict(cls, value, parser=None):
        obj = Location(value, parser=parser)
        return obj

    def __init__(self, value, loader=None, parser=None):
        self.value = value
        self.loader = loader
        self.parser = parser

    def load(self):
        data = self.loader(self.value) if self.loader else self.value
        return self.parser(data) if self.parser else data


class ConfigData(object):
    ''' ConfigData is effectively a simplified, read-only, ChainMap'''

    def __init__(self, maps):
        self.maps = maps

    def get(self, key, default=None):
        try:
            return self.value_for_key(key)
        except KeyError:
            return default

    def __getattr__(self, key):

        try:
            return self[key]
        except KeyError:
            raise AttributeError(key)

    def __getitem__(self, key):
        result = self.value_for_key(key)

        try:
            return AttributeDict(result)
        except (TypeError, ValueError):
            return result

    def value_for_key(self, key):
        maps = self.maps
        for each in maps:
            try:
                return each[key]
            except KeyError:
                pass

        raise KeyError(key)


class Config(object):
    def __init__(self, defaults=None, chain=False):
        self.defaults = {} if defaults is None else defaults
        self.chain = chain
        self.locations = []

    @property
    def data(self):
        try:
            return self._data
        except AttributeError:
            self._data = self.load()
            return self._data

    def extend(self, config):
        self.data.maps.append(config)

    def load(self):
        if self.chain:
            return self.load_chain(self.locations)
        else:
            return self.load_first(self.locations)

    def load_chain(self, locations):
        data = deque()
        data.append(self.defaults)
        [data.appendleft(x) for x in load_locations(locations)]

        return ConfigData(data)

    def load_first(self, locations):
        data = deque()
        for x in load_locations(locations):
            data.append(x)
            return ConfigData(data)

        data.append(self.defaults)
        return ConfigData(data)


class ConfigManager(object):
    pass



import sys
_ver = sys.version_info

is_py2 = (_ver[0] == 2)
is_py3 = (_ver[0] == 3)

if is_py2:
    from ConfigParser import SafeConfigParser
    from cStringIO import StringIO
    string_type = basestring
    iterkeys = dict.iterkeys

elif is_py3:
    from configparser import SafeConfigParser
    from io import StringIO
    string_type = str
    iterkeys = dict.keys


# backport of import lib
# https://pypi.python.org/pypi/importlib/1.0.2
if is_py2:
    def _resolve_name(name, package, level):
        """Return the absolute name of the module to be imported."""
        if not hasattr(package, 'rindex'):
            raise ValueError("'package' not set to a string")
        dot = len(package)
        for x in xrange(level, 1, -1):
            try:
                dot = package.rindex('.', 0, dot)
            except ValueError:
                raise ValueError("attempted relative import beyond top-level "
                                  "package")
        return "%s.%s" % (package[:dot], name)

    def import_module(name, package=None):
        """Import a module.

        The 'package' argument is required when performing a relative import. It
        specifies the package to use as the anchor point from which to resolve the
        relative import to an absolute import.

        """
        if name.startswith('.'):
            if not package:
                raise TypeError("relative imports require the 'package' argument")
            level = 0
            for character in name:
                if character != '.':
                    break
                level += 1
            name = _resolve_name(name[level:], package, level)
        __import__(name)
        return sys.modules[name]



#!/usr/bin/env python
import os
from setuptools import setup

with open('README.md') as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()


def strip_comments(l):
    return l.split('#', 1)[0].strip()


def reqs(*f):
    return list(filter(None, [strip_comments(l) for l in open(
        os.path.join(os.getcwd(), 'requirements', *f)).readlines()]))

install_requires = reqs('default.txt')
tests_require = ['nose']

packages = [
    'confypy'
]

setup(
    name='confypy',
    version='0.2.4',
    description='Configuration Loading Utility',
    long_description=readme,
    author='Adam Venturella',
    author_email='aventurella@gmail.com',
    url='https://github.com/aventurella/confypy',
    license=license,
    packages=packages,
    package_data={'': ['LICENSE']},
    include_package_data=True,
    install_requires=install_requires,
    tests_require=tests_require,
    package_dir={'confypy': 'confypy'},
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
    ),
)



import os
from .exceptions import LoadError
from .compat import iterkeys


try:
    from importlib import import_module as import_module
except ImportError:
    from .compat import import_module


def load_locations(locations):
    for each in locations:
        try:
            data = each.load()
        except LoadError:
            continue

        yield data


def load_env_python(key):
    path = os.getenv(key, None)
    return load_python(path)


def load_python(path):
    try:
        module = import_module(path).__dict__
    except ImportError:
        raise LoadError

    data = {}

    for key in [x for x in iterkeys(module)
                if not x.startswith('__') and
                x not in('os', 'sys')]:
        data[key] = module[key]

    return data


def load_env_keys(keys):
    data = {}
    for each in keys:
        try:
            data[each] = os.environ[each]
        except KeyError:
            pass
    return data


def load_env_path(key):
    path = os.getenv(key, None)

    if path:
        return load_file(path)

    raise LoadError


def load_file(path):
    try:
        with open(os.path.expanduser(path)) as f:
            return f.read()
    except IOError:
        raise LoadError
